@@ -28,6 +28,7 @@
 #include "CppUTest/CommandLineTestRunner.h"
 #include "CppUTest/TestPlugin.h"
 #include "CppUTest/TestRegistry.h"
+#include "CppUTestExt/IEEE754ExceptionsPlugin.h"
 #include "CppUTestExt/MockSupportPlugin.h"
 
 class MyDummyComparator : public MockNamedValueComparator
@@ -48,9 +49,11 @@ int main(int ac, char** av)
 {
     MyDummyComparator dummyComparator;
     MockSupportPlugin mockPlugin;
+    IEEE754ExceptionsPlugin ieee754Plugin;
 
     mockPlugin.installComparator("MyDummyType", dummyComparator);
     TestRegistry::getCurrentRegistry()->installPlugin(&mockPlugin);
+    TestRegistry::getCurrentRegistry()->installPlugin(&ieee754Plugin);
     return CommandLineTestRunner::RunAllTests(ac, av);
 }
 
