@@ -55,13 +55,15 @@ public:
     ~SimpleString();
 
     SimpleString& operator=(const SimpleString& other);
-    SimpleString operator+(const SimpleString&);
+    SimpleString operator+(const SimpleString&) const;
     SimpleString& operator+=(const SimpleString&);
     SimpleString& operator+=(const char*);
 
-    char at(int pos) const;
-    int find(char ch) const;
-    int findFrom(size_t starting_position, char ch) const;
+    static const size_t npos = (size_t) -1;
+
+    char at(size_t pos) const;
+    size_t find(char ch) const;
+    size_t findFrom(size_t starting_position, char ch) const;
     bool contains(const SimpleString& other) const;
     bool containsNoCase(const SimpleString& other) const;
     bool startsWith(const SimpleString& other) const;
@@ -76,6 +78,7 @@ public:
     void replace(const char* to, const char* with);
 
     SimpleString lowerCase() const;
+    SimpleString subString(size_t beginPos) const;
     SimpleString subString(size_t beginPos, size_t amount) const;
     SimpleString subStringFromTill(char startChar, char lastExcludedChar) const;
     void copyToBuffer(char* buffer, size_t bufferSize) const;
@@ -89,23 +92,36 @@ public:
     static TestMemoryAllocator* getStringAllocator();
     static void setStringAllocator(TestMemoryAllocator* allocator);
 
-    static char* allocStringBuffer(size_t size);
     static int AtoI(const char*str);
+    static unsigned AtoU(const char*str);
     static int StrCmp(const char* s1, const char* s2);
     static size_t StrLen(const char*);
     static int StrNCmp(const char* s1, const char* s2, size_t n);
     static char* StrNCpy(char* s1, const char* s2, size_t n);
-    static char* StrStr(const char* s1, const char* s2);
+    static const char* StrStr(const char* s1, const char* s2);
     static char ToLower(char ch);
     static int MemCmp(const void* s1, const void *s2, size_t n);
-    static void deallocStringBuffer(char* str);
+    static char* allocStringBuffer(size_t size, const char* file, size_t line);
+    static void deallocStringBuffer(char* str, size_t size, const char* file, size_t line);
 private:
+
+    const char* getBuffer() const;
+
+    void deallocateInternalBuffer();
+    void setInternalBufferAsEmptyString();
+    void setInternalBufferToNewBuffer(size_t bufferSize);
+    void setInternalBufferTo(char* buffer, size_t bufferSize);
+    void copyBufferToNewInternalBuffer(const char* otherBuffer);
+    void copyBufferToNewInternalBuffer(const char* otherBuffer, size_t bufferSize);
+    void copyBufferToNewInternalBuffer(const SimpleString& otherBuffer);
+
     char *buffer_;
+    size_t bufferSize_;
 
     static TestMemoryAllocator* stringAllocator_;
 
     char* getEmptyString() const;
-    static char* copyToNewBuffer(const char* bufferToCopy, size_t bufferSize=0);
+    static char* copyToNewBuffer(const char* bufferToCopy, size_t bufferSize);
     static bool isDigit(char ch);
     static bool isSpace(char ch);
     static bool isUpper(char ch);
@@ -131,8 +147,42 @@ private:
     SimpleStringCollection(SimpleStringCollection&);
 };
 
+class GlobalSimpleStringAllocatorStash
+{
+public:
+    GlobalSimpleStringAllocatorStash();
+    void save();
+    void restore();
+private:
+    TestMemoryAllocator* originalAllocator_;
+};
+
+class MemoryAccountant;
+class AccountingTestMemoryAllocator;
+
+class GlobalSimpleStringMemoryAccountant
+{
+public:
+    GlobalSimpleStringMemoryAccountant();
+    ~GlobalSimpleStringMemoryAccountant();
+
+    void useCacheSizes(size_t cacheSizes[], size_t length);
+
+    void start();
+    void stop();
+    SimpleString report();
+
+    AccountingTestMemoryAllocator* getAllocator();
+private:
+    void restoreAllocator();
+
+    AccountingTestMemoryAllocator* allocator_;
+    MemoryAccountant* accountant_;
+};
+
 SimpleString StringFrom(bool value);
 SimpleString StringFrom(const void* value);
+SimpleString StringFrom(void (*value)());
 SimpleString StringFrom(char value);
 SimpleString StringFrom(const char *value);
 SimpleString StringFromOrNull(const char * value);
@@ -140,15 +190,43 @@ SimpleString StringFrom(int value);
 SimpleString StringFrom(unsigned int value);
 SimpleString StringFrom(long value);
 SimpleString StringFrom(unsigned long value);
+SimpleString StringFrom(cpputest_longlong value);
+SimpleString StringFrom(cpputest_ulonglong value);
+SimpleString HexStringFrom(unsigned int value);
+SimpleString HexStringFrom(int value);
+SimpleString HexStringFrom(signed char value);
 SimpleString HexStringFrom(long value);
 SimpleString HexStringFrom(unsigned long value);
+SimpleString HexStringFrom(cpputest_longlong value);
+SimpleString HexStringFrom(cpputest_ulonglong value);
 SimpleString HexStringFrom(const void* value);
+SimpleString HexStringFrom(void (*value)());
 SimpleString StringFrom(double value, int precision = 6);
 SimpleString StringFrom(const SimpleString& other);
-SimpleString StringFromFormat(const char* format, ...) __check_format__(printf, 1, 2);
+SimpleString StringFromFormat(const char* format, ...) _check_format_(printf, 1, 2);
 SimpleString VStringFromFormat(const char* format, va_list args);
 SimpleString StringFromBinary(const unsigned char* value, size_t size);
 SimpleString StringFromBinaryOrNull(const unsigned char* value, size_t size);
+SimpleString StringFromBinaryWithSize(const unsigned char* value, size_t size);
+SimpleString StringFromBinaryWithSizeOrNull(const unsigned char* value, size_t size);
+SimpleString StringFromMaskedBits(unsigned long value, unsigned long mask, size_t byteCount);
+SimpleString StringFromOrdinalNumber(unsigned int number);
+SimpleString BracketsFormattedHexStringFrom(int value);
+SimpleString BracketsFormattedHexStringFrom(unsigned int value);
+SimpleString BracketsFormattedHexStringFrom(long value);
+SimpleString BracketsFormattedHexStringFrom(unsigned long value);
+SimpleString BracketsFormattedHexStringFrom(cpputest_longlong value);
+SimpleString BracketsFormattedHexStringFrom(cpputest_ulonglong value);
+SimpleString BracketsFormattedHexStringFrom(signed char value);
+SimpleString BracketsFormattedHexString(SimpleString hexString);
+
+/*
+ * ARM compiler has only partial support for C++11.
+ * Specifically nullptr_t is not officially supported
+ */
+#if __cplusplus > 199711L && !defined __arm__ && CPPUTEST_USE_STD_CPP_LIB
+SimpleString StringFrom(const std::nullptr_t value);
+#endif
 
 #if CPPUTEST_USE_STD_CPP_LIB
 
