@@ -44,80 +44,83 @@ private:
     ScopedMutexLock lock;
 };
 
-static void* threadsafe_mem_leak_malloc(size_t size, const char* file, int line)
+static void* threadsafe_mem_leak_malloc(size_t size, const char* file, size_t line)
 {
     MemLeakScopedMutex lock;
     return MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentMallocAllocator(), size, file, line, true);
 }
 
-static void threadsafe_mem_leak_free(void* buffer, const char* file, int line)
+static void threadsafe_mem_leak_free(void* buffer, const char* file, size_t line)
 {
     MemLeakScopedMutex lock;
     MemoryLeakWarningPlugin::getGlobalDetector()->invalidateMemory((char*) buffer);
     MemoryLeakWarningPlugin::getGlobalDetector()->deallocMemory(getCurrentMallocAllocator(), (char*) buffer, file, line, true);
 }
 
-static void* threadsafe_mem_leak_realloc(void* memory, size_t size, const char* file, int line)
+static void* threadsafe_mem_leak_realloc(void* memory, size_t size, const char* file, size_t line)
 {
     MemLeakScopedMutex lock;
     return MemoryLeakWarningPlugin::getGlobalDetector()->reallocMemory(getCurrentMallocAllocator(), (char*) memory, size, file, line, true);
 }
 
 
-static void* mem_leak_malloc(size_t size, const char* file, int line)
+static void* mem_leak_malloc(size_t size, const char* file, size_t line)
 {
     return MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentMallocAllocator(), size, file, line, true);
 }
 
-static void mem_leak_free(void* buffer, const char* file, int line)
+static void mem_leak_free(void* buffer, const char* file, size_t line)
 {
     MemoryLeakWarningPlugin::getGlobalDetector()->invalidateMemory((char*) buffer);
     MemoryLeakWarningPlugin::getGlobalDetector()->deallocMemory(getCurrentMallocAllocator(), (char*) buffer, file, line, true);
 }
 
-static void* mem_leak_realloc(void* memory, size_t size, const char* file, int line)
+static void* mem_leak_realloc(void* memory, size_t size, const char* file, size_t line)
 {
     return MemoryLeakWarningPlugin::getGlobalDetector()->reallocMemory(getCurrentMallocAllocator(), (char*) memory, size, file, line, true);
 }
 
 #endif
 
-static void* normal_malloc(size_t size, const char*, int)
+static void* normal_malloc(size_t size, const char*, size_t)
 {
     return PlatformSpecificMalloc(size);
 }
 
-static void* normal_realloc(void* memory, size_t size, const char*, int)
+static void* normal_realloc(void* memory, size_t size, const char*, size_t)
 {
     return PlatformSpecificRealloc(memory, size);
 }
 
-static void normal_free(void* buffer, const char*, int)
+static void normal_free(void* buffer, const char*, size_t)
 {
     PlatformSpecificFree(buffer);
 }
 
 #if CPPUTEST_USE_MEM_LEAK_DETECTION
-static void *(*malloc_fptr)(size_t size, const char* file, int line) = mem_leak_malloc;
-static void (*free_fptr)(void* mem, const char* file, int line) = mem_leak_free;
-static void*(*realloc_fptr)(void* memory, size_t size, const char* file, int line) = mem_leak_realloc;
+static void *(*malloc_fptr)(size_t size, const char* file, size_t line) = mem_leak_malloc;
+static void (*free_fptr)(void* mem, const char* file, size_t line) = mem_leak_free;
+static void*(*realloc_fptr)(void* memory, size_t size, const char* file, size_t line) = mem_leak_realloc;
+static void *(*saved_malloc_fptr)(size_t size, const char* file, size_t line) = mem_leak_malloc;
+static void (*saved_free_fptr)(void* mem, const char* file, size_t line) = mem_leak_free;
+static void*(*saved_realloc_fptr)(void* memory, size_t size, const char* file, size_t line) = mem_leak_realloc;
 #else
-static void *(*malloc_fptr)(size_t size, const char* file, int line) = normal_malloc;
-static void (*free_fptr)(void* mem, const char* file, int line) = normal_free;
-static void*(*realloc_fptr)(void* memory, size_t size, const char* file, int line) = normal_realloc;
+static void *(*malloc_fptr)(size_t size, const char* file, size_t line) = normal_malloc;
+static void (*free_fptr)(void* mem, const char* file, size_t line) = normal_free;
+static void*(*realloc_fptr)(void* memory, size_t size, const char* file, size_t line) = normal_realloc;
 #endif
 
-void* cpputest_malloc_location_with_leak_detection(size_t size, const char* file, int line)
+void* cpputest_malloc_location_with_leak_detection(size_t size, const char* file, size_t line)
 {
     return malloc_fptr(size, file, line);
 }
 
-void* cpputest_realloc_location_with_leak_detection(void* memory, size_t size, const char* file, int line)
+void* cpputest_realloc_location_with_leak_detection(void* memory, size_t size, const char* file, size_t line)
 {
     return realloc_fptr(memory, size, file, line);
 }
 
-void cpputest_free_location_with_leak_detection(void* buffer, const char* file, int line)
+void cpputest_free_location_with_leak_detection(void* buffer, const char* file, size_t line)
 {
     free_fptr(buffer, file, line);
 }
@@ -128,7 +131,7 @@ void cpputest_free_location_with_leak_detection(void* buffer, const char* file,
 #undef new
 
 #if CPPUTEST_USE_STD_CPP_LIB
-#define UT_THROW_BAD_ALLOC_WHEN_NULL(memory) if (memory == NULL) throw std::bad_alloc();
+#define UT_THROW_BAD_ALLOC_WHEN_NULL(memory) if (memory == NULLPTR) throw std::bad_alloc()
 #else
 #define UT_THROW_BAD_ALLOC_WHEN_NULL(memory)
 #endif
@@ -147,10 +150,10 @@ static void* threadsafe_mem_leak_operator_new_nothrow (size_t size) UT_NOTHROW
     return MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewAllocator(), size);
 }
 
-static void* threadsafe_mem_leak_operator_new_debug (size_t size, const char* file, int line) UT_THROW(std::bad_alloc)
+static void* threadsafe_mem_leak_operator_new_debug (size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc)
 {
     MemLeakScopedMutex lock;
-    void *memory = MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewAllocator(), size, (char*) file, line);
+    void *memory = MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewAllocator(), size, file, line);
     UT_THROW_BAD_ALLOC_WHEN_NULL(memory);
     return memory;
 }
@@ -169,10 +172,10 @@ static void* threadsafe_mem_leak_operator_new_array_nothrow (size_t size) UT_NOT
     return MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewArrayAllocator(), size);
 }
 
-static void* threadsafe_mem_leak_operator_new_array_debug (size_t size, const char* file, int line) UT_THROW(std::bad_alloc)
+static void* threadsafe_mem_leak_operator_new_array_debug (size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc)
 {
     MemLeakScopedMutex lock;
-    void* memory = MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewArrayAllocator(), size, (char*) file, line);
+    void* memory = MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewArrayAllocator(), size, file, line);
     UT_THROW_BAD_ALLOC_WHEN_NULL(memory);
     return memory;
 }
@@ -204,9 +207,9 @@ static void* mem_leak_operator_new_nothrow (size_t size) UT_NOTHROW
     return MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewAllocator(), size);
 }
 
-static void* mem_leak_operator_new_debug (size_t size, const char* file, int line) UT_THROW(std::bad_alloc)
+static void* mem_leak_operator_new_debug (size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc)
 {
-    void *memory = MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewAllocator(), size, (char*) file, line);
+    void *memory = MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewAllocator(), size, file, line);
     UT_THROW_BAD_ALLOC_WHEN_NULL(memory);
     return memory;
 }
@@ -223,9 +226,9 @@ static void* mem_leak_operator_new_array_nothrow (size_t size) UT_NOTHROW
     return MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewArrayAllocator(), size);
 }
 
-static void* mem_leak_operator_new_array_debug (size_t size, const char* file, int line) UT_THROW(std::bad_alloc)
+static void* mem_leak_operator_new_array_debug (size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc)
 {
-    void* memory = MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewArrayAllocator(), size, (char*) file, line);
+    void* memory = MemoryLeakWarningPlugin::getGlobalDetector()->allocMemory(getCurrentNewArrayAllocator(), size, file, line);
     UT_THROW_BAD_ALLOC_WHEN_NULL(memory);
     return memory;
 }
@@ -254,7 +257,7 @@ static void* normal_operator_new_nothrow (size_t size) UT_NOTHROW
     return PlatformSpecificMalloc(size);
 }
 
-static void* normal_operator_new_debug (size_t size, const char* /*file*/, int /*line*/) UT_THROW(std::bad_alloc)
+static void* normal_operator_new_debug (size_t size, const char* /*file*/, size_t /*line*/) UT_THROW(std::bad_alloc)
 {
     void* memory = PlatformSpecificMalloc(size);
     UT_THROW_BAD_ALLOC_WHEN_NULL(memory);
@@ -273,7 +276,7 @@ static void* normal_operator_new_array_nothrow (size_t size) UT_NOTHROW
     return PlatformSpecificMalloc(size);
 }
 
-static void* normal_operator_new_array_debug (size_t size, const char* /*file*/, int /*line*/) UT_THROW(std::bad_alloc)
+static void* normal_operator_new_array_debug (size_t size, const char* /*file*/, size_t /*line*/) UT_THROW(std::bad_alloc)
 {
     void* memory = PlatformSpecificMalloc(size);
     UT_THROW_BAD_ALLOC_WHEN_NULL(memory);
@@ -292,19 +295,29 @@ static void normal_operator_delete_array (void* mem) UT_NOTHROW
 
 static void *(*operator_new_fptr)(size_t size) UT_THROW(std::bad_alloc) = mem_leak_operator_new;
 static void *(*operator_new_nothrow_fptr)(size_t size) UT_NOTHROW = mem_leak_operator_new_nothrow;
-static void *(*operator_new_debug_fptr)(size_t size, const char* file, int line) UT_THROW(std::bad_alloc) = mem_leak_operator_new_debug;
+static void *(*operator_new_debug_fptr)(size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc) = mem_leak_operator_new_debug;
 static void *(*operator_new_array_fptr)(size_t size) UT_THROW(std::bad_alloc) = mem_leak_operator_new_array;
 static void *(*operator_new_array_nothrow_fptr)(size_t size) UT_NOTHROW = mem_leak_operator_new_array_nothrow;
-static void *(*operator_new_array_debug_fptr)(size_t size, const char* file, int line) UT_THROW(std::bad_alloc) = mem_leak_operator_new_array_debug;
+static void *(*operator_new_array_debug_fptr)(size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc) = mem_leak_operator_new_array_debug;
 static void (*operator_delete_fptr)(void* mem) UT_NOTHROW = mem_leak_operator_delete;
 static void (*operator_delete_array_fptr)(void* mem) UT_NOTHROW = mem_leak_operator_delete_array;
 
+static void *(*saved_operator_new_fptr)(size_t size) UT_THROW(std::bad_alloc) = mem_leak_operator_new;
+static void *(*saved_operator_new_nothrow_fptr)(size_t size) UT_NOTHROW = mem_leak_operator_new_nothrow;
+static void *(*saved_operator_new_debug_fptr)(size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc) = mem_leak_operator_new_debug;
+static void *(*saved_operator_new_array_fptr)(size_t size) UT_THROW(std::bad_alloc) = mem_leak_operator_new_array;
+static void *(*saved_operator_new_array_nothrow_fptr)(size_t size) UT_NOTHROW = mem_leak_operator_new_array_nothrow;
+static void *(*saved_operator_new_array_debug_fptr)(size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc) = mem_leak_operator_new_array_debug;
+static void (*saved_operator_delete_fptr)(void* mem) UT_NOTHROW = mem_leak_operator_delete;
+static void (*saved_operator_delete_array_fptr)(void* mem) UT_NOTHROW = mem_leak_operator_delete_array;
+static int save_counter = 0;
+
 void* operator new(size_t size) UT_THROW(std::bad_alloc)
 {
     return operator_new_fptr(size);
 }
 
-void* operator new(size_t size, const char* file, int line) UT_THROW(std::bad_alloc)
+void* operator new(size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc)
 {
     return operator_new_debug_fptr(size, file, line);
 }
@@ -314,22 +327,24 @@ void operator delete(void* mem) UT_NOTHROW
     operator_delete_fptr(mem);
 }
 
-void operator delete(void* mem, const char*, int) UT_NOTHROW
+void operator delete(void* mem, const char*, size_t) UT_NOTHROW
 {
     operator_delete_fptr(mem);
 }
 
+#if __cplusplus >= 201402L
 void operator delete (void* mem, size_t) UT_NOTHROW
 {
     operator_delete_fptr(mem);
 }
+#endif
 
 void* operator new[](size_t size) UT_THROW(std::bad_alloc)
 {
     return operator_new_array_fptr(size);
 }
 
-void* operator new [](size_t size, const char* file, int line) UT_THROW(std::bad_alloc)
+void* operator new [](size_t size, const char* file, size_t line) UT_THROW(std::bad_alloc)
 {
     return operator_new_array_debug_fptr(size, file, line);
 }
@@ -339,15 +354,17 @@ void operator delete[](void* mem) UT_NOTHROW
      operator_delete_array_fptr(mem);
 }
 
-void operator delete[](void* mem, const char*, int) UT_NOTHROW
+void operator delete[](void* mem, const char*, size_t) UT_NOTHROW
 {
      operator_delete_array_fptr(mem);
 }
 
+#if __cplusplus >= 201402L
 void operator delete[] (void* mem, size_t) UT_NOTHROW
 {
      operator_delete_array_fptr(mem);
 }
+#endif
 
 #if CPPUTEST_USE_STD_CPP_LIB
 
@@ -356,11 +373,21 @@ void* operator new(size_t size, const std::nothrow_t&) UT_NOTHROW
     return operator_new_nothrow_fptr(size);
 }
 
+void operator delete(void* mem, const std::nothrow_t&) UT_NOTHROW
+{
+    operator_delete_fptr(mem);
+}
+
 void* operator new[](size_t size, const std::nothrow_t&) UT_NOTHROW
 {
     return operator_new_array_nothrow_fptr(size);
 }
 
+void operator delete[](void* mem, const std::nothrow_t&) UT_NOTHROW
+{
+    operator_delete_array_fptr(mem);
+}
+
 #else
 
 /* Have a similar method. This avoid unused operator_new_nothrow_fptr warning */
@@ -399,7 +426,7 @@ void MemoryLeakWarningPlugin::turnOffNewDeleteOverloads()
 #endif
 }
 
-void MemoryLeakWarningPlugin::turnOnNewDeleteOverloads()
+void MemoryLeakWarningPlugin::turnOnDefaultNotThreadSafeNewDeleteOverloads()
 {
 #if CPPUTEST_USE_MEM_LEAK_DETECTION
     operator_new_fptr = mem_leak_operator_new;
@@ -416,15 +443,6 @@ void MemoryLeakWarningPlugin::turnOnNewDeleteOverloads()
 #endif
 }
 
-bool MemoryLeakWarningPlugin::areNewDeleteOverloaded()
-{
-#if CPPUTEST_USE_MEM_LEAK_DETECTION
-    return operator_new_fptr == mem_leak_operator_new;
-#else
-    return false;
-#endif
-}
-
 void MemoryLeakWarningPlugin::turnOnThreadSafeNewDeleteOverloads()
 {
 #if CPPUTEST_USE_MEM_LEAK_DETECTION
@@ -442,6 +460,52 @@ void MemoryLeakWarningPlugin::turnOnThreadSafeNewDeleteOverloads()
 #endif
 }
 
+bool MemoryLeakWarningPlugin::areNewDeleteOverloaded()
+{
+#if CPPUTEST_USE_MEM_LEAK_DETECTION
+    return operator_new_fptr == mem_leak_operator_new || operator_new_fptr == threadsafe_mem_leak_operator_new;
+#else
+    return false;
+#endif
+}
+
+void MemoryLeakWarningPlugin::saveAndDisableNewDeleteOverloads()
+{
+#if CPPUTEST_USE_MEM_LEAK_DETECTION
+    if (++save_counter > 1) return;
+    saved_operator_new_fptr = operator_new_fptr;
+    saved_operator_new_nothrow_fptr = operator_new_nothrow_fptr;
+    saved_operator_new_debug_fptr = operator_new_debug_fptr;
+    saved_operator_new_array_fptr = operator_new_array_fptr;
+    saved_operator_new_array_nothrow_fptr = operator_new_array_nothrow_fptr;
+    saved_operator_new_array_debug_fptr = operator_new_array_debug_fptr;
+    saved_operator_delete_fptr = operator_delete_fptr;
+    saved_operator_delete_array_fptr = operator_delete_array_fptr;
+    saved_malloc_fptr = malloc_fptr;
+    saved_realloc_fptr = realloc_fptr;
+    saved_free_fptr = free_fptr;
+    turnOffNewDeleteOverloads();
+#endif
+}
+
+void MemoryLeakWarningPlugin::restoreNewDeleteOverloads()
+{
+#if CPPUTEST_USE_MEM_LEAK_DETECTION
+    if (--save_counter > 0) return;
+    operator_new_fptr = saved_operator_new_fptr;
+    operator_new_nothrow_fptr = saved_operator_new_nothrow_fptr;
+    operator_new_debug_fptr = saved_operator_new_debug_fptr;
+    operator_new_array_fptr = saved_operator_new_array_fptr;
+    operator_new_array_nothrow_fptr = saved_operator_new_array_nothrow_fptr;
+    operator_new_array_debug_fptr = saved_operator_new_array_debug_fptr;
+    operator_delete_fptr = saved_operator_delete_fptr;
+    operator_delete_array_fptr = saved_operator_delete_array_fptr;
+    malloc_fptr = saved_malloc_fptr;
+    realloc_fptr = saved_realloc_fptr;
+    free_fptr = saved_free_fptr;
+#endif
+}
+
 void crash_on_allocation_number(unsigned alloc_number)
 {
     static CrashOnAllocationAllocator crashAllocator;
@@ -454,30 +518,29 @@ void crash_on_allocation_number(unsigned alloc_number)
 class MemoryLeakWarningReporter: public MemoryLeakFailure
 {
 public:
-    virtual ~MemoryLeakWarningReporter()
+    virtual ~MemoryLeakWarningReporter() _destructor_override
     {
     }
 
-    virtual void fail(char* fail_string)
+    virtual void fail(char* fail_string) _override
     {
         UtestShell* currentTest = UtestShell::getCurrent();
         currentTest->failWith(FailFailure(currentTest, currentTest->getName().asCharString(), currentTest->getLineNumber(), fail_string), TestTerminatorWithoutExceptions());
     } // LCOV_EXCL_LINE
 };
 
-static MemoryLeakFailure* globalReporter = 0;
-static MemoryLeakDetector* globalDetector = 0;
+static MemoryLeakFailure* globalReporter = NULLPTR;
+static MemoryLeakDetector* globalDetector = NULLPTR;
 
 MemoryLeakDetector* MemoryLeakWarningPlugin::getGlobalDetector()
 {
-    if (globalDetector == 0) {
-        bool newDeleteOverloaded = areNewDeleteOverloaded();
-        turnOffNewDeleteOverloads();
+    if (globalDetector == NULLPTR) {
+        saveAndDisableNewDeleteOverloads();
 
         globalReporter = new MemoryLeakWarningReporter;
         globalDetector = new MemoryLeakDetector(globalReporter);
 
-        if (newDeleteOverloaded) turnOnNewDeleteOverloads();
+        restoreNewDeleteOverloads();
     }
     return globalDetector;
 }
@@ -503,11 +566,11 @@ void MemoryLeakWarningPlugin::destroyGlobalDetector()
     turnOffNewDeleteOverloads();
     delete globalDetector;
     delete globalReporter;
-    globalDetector = NULL;
+    globalDetector = NULLPTR;
 }
 
 
-MemoryLeakWarningPlugin* MemoryLeakWarningPlugin::firstPlugin_ = 0;
+MemoryLeakWarningPlugin* MemoryLeakWarningPlugin::firstPlugin_ = NULLPTR;
 
 MemoryLeakWarningPlugin* MemoryLeakWarningPlugin::getFirstPlugin()
 {
@@ -524,7 +587,7 @@ void MemoryLeakWarningPlugin::ignoreAllLeaksInTest()
     ignoreAllWarnings_ = true;
 }
 
-void MemoryLeakWarningPlugin::expectLeaksInTest(int n)
+void MemoryLeakWarningPlugin::expectLeaksInTest(size_t n)
 {
     expectedLeaks_ = n;
 }
@@ -532,7 +595,7 @@ void MemoryLeakWarningPlugin::expectLeaksInTest(int n)
 MemoryLeakWarningPlugin::MemoryLeakWarningPlugin(const SimpleString& name, MemoryLeakDetector* localDetector) :
     TestPlugin(name), ignoreAllWarnings_(false), destroyGlobalDetectorAndTurnOfMemoryLeakDetectionInDestructor_(false), expectedLeaks_(0)
 {
-    if (firstPlugin_ == 0) firstPlugin_ = this;
+    if (firstPlugin_ == NULLPTR) firstPlugin_ = this;
 
     if (localDetector) memLeakDetector_ = localDetector;
     else memLeakDetector_ = getGlobalDetector();
@@ -557,20 +620,24 @@ void MemoryLeakWarningPlugin::preTestAction(UtestShell& /*test*/, TestResult& re
 void MemoryLeakWarningPlugin::postTestAction(UtestShell& test, TestResult& result)
 {
     memLeakDetector_->stopChecking();
-    int leaks = memLeakDetector_->totalMemoryLeaks(mem_leak_period_checking);
+    size_t leaks = memLeakDetector_->totalMemoryLeaks(mem_leak_period_checking);
 
     if (!ignoreAllWarnings_ && expectedLeaks_ != leaks && failureCount_ == result.getFailureCount()) {
-        TestFailure f(&test, memLeakDetector_->report(mem_leak_period_checking));
-        result.addFailure(f);
+        if(MemoryLeakWarningPlugin::areNewDeleteOverloaded()) {
+            TestFailure f(&test, memLeakDetector_->report(mem_leak_period_checking));
+            result.addFailure(f);
+        } else if(expectedLeaks_ > 0) {
+            result.print(StringFromFormat("Warning: Expected %d leak(s), but leak detection was disabled", (int) expectedLeaks_).asCharString());
+        }
     }
     memLeakDetector_->markCheckingPeriodLeaksAsNonCheckingPeriod();
     ignoreAllWarnings_ = false;
     expectedLeaks_ = 0;
 }
 
-const char* MemoryLeakWarningPlugin::FinalReport(int toBeDeletedLeaks)
+const char* MemoryLeakWarningPlugin::FinalReport(size_t toBeDeletedLeaks)
 {
-    int leaks = memLeakDetector_->totalMemoryLeaks(mem_leak_period_enabled);
+    size_t leaks = memLeakDetector_->totalMemoryLeaks(mem_leak_period_enabled);
     if (leaks != toBeDeletedLeaks) return memLeakDetector_->report(mem_leak_period_enabled);
     return "";
 }
