@@ -49,7 +49,7 @@ public:
         TestTerminatorWithoutExceptions::exitCurrentTest();
     } // LCOV_EXCL_LINE
     // LCOV_EXCL_START
-    virtual ~MockFailureReporterTestTerminatorForInCOnlyCode()
+    virtual ~MockFailureReporterTestTerminatorForInCOnlyCode() _destructor_override
     {
     }
     // LCOV_EXCL_STOP
@@ -69,9 +69,9 @@ public:
 
 };
 
-static MockSupport* currentMockSupport = NULL;
-static MockExpectedCall* expectedCall = NULL;
-static MockActualCall* actualCall = NULL;
+static MockSupport* currentMockSupport = NULLPTR;
+static MockExpectedCall* expectedCall = NULLPTR;
+static MockActualCall* actualCall = NULLPTR;
 static MockFailureReporterForInCOnlyCode failureReporterForC;
 
 class MockCFunctionComparatorNode : public MockNamedValueComparator
@@ -79,7 +79,7 @@ class MockCFunctionComparatorNode : public MockNamedValueComparator
 public:
     MockCFunctionComparatorNode(MockCFunctionComparatorNode* next, MockTypeEqualFunction_c equal, MockTypeValueToStringFunction_c toString)
         : next_(next), equal_(equal), toString_(toString) {}
-    virtual ~MockCFunctionComparatorNode() {}
+    virtual ~MockCFunctionComparatorNode() _destructor_override {}
 
     virtual bool isEqual(const void* object1, const void* object2) _override
     {
@@ -95,14 +95,14 @@ public:
     MockTypeValueToStringFunction_c toString_;
 };
 
-static MockCFunctionComparatorNode* comparatorList_ = NULL;
+static MockCFunctionComparatorNode* comparatorList_ = NULLPTR;
 
 class MockCFunctionCopierNode : public MockNamedValueCopier
 {
 public:
     MockCFunctionCopierNode(MockCFunctionCopierNode* next, MockTypeCopyFunction_c copier)
         : next_(next), copier_(copier) {}
-    virtual ~MockCFunctionCopierNode() {}
+    virtual ~MockCFunctionCopierNode() _destructor_override {}
 
     virtual void copy(void* dst, const void* src) _override
     {
@@ -113,14 +113,14 @@ public:
     MockTypeCopyFunction_c copier_;
 };
 
-static MockCFunctionCopierNode* copierList_ = NULL;
+static MockCFunctionCopierNode* copierList_ = NULLPTR;
 
 extern "C" {
 
 void strictOrder_c();
 MockExpectedCall_c* expectOneCall_c(const char* name);
 void expectNoCall_c(const char* name);
-MockExpectedCall_c* expectNCalls_c(const int number, const char* name);
+MockExpectedCall_c* expectNCalls_c(const unsigned int number, const char* name);
 MockActualCall_c* actualCall_c(const char* name);
 void disable_c();
 void enable_c();
@@ -134,6 +134,7 @@ void setPointerData_c(const char* name, void* value);
 void setConstPointerData_c(const char* name, const void* value);
 void setFunctionPointerData_c(const char* name, void (*value)());
 void setDataObject_c(const char* name, const char* type, void* value);
+void setDataConstObject_c(const char* name, const char* type, const void* value);
 MockValue_c getData_c(const char* name);
 int hasReturnValue_c();
 
@@ -147,7 +148,10 @@ MockExpectedCall_c* withIntParameters_c(const char* name, int value);
 MockExpectedCall_c* withUnsignedIntParameters_c(const char* name, unsigned int value);
 MockExpectedCall_c* withLongIntParameters_c(const char* name, long int value);
 MockExpectedCall_c* withUnsignedLongIntParameters_c(const char* name, unsigned long int value);
+MockExpectedCall_c* withLongLongIntParameters_c(const char* name, cpputest_longlong value);
+MockExpectedCall_c* withUnsignedLongLongIntParameters_c(const char* name, cpputest_ulonglong value);
 MockExpectedCall_c* withDoubleParameters_c(const char* name, double value);
+MockExpectedCall_c* withDoubleParametersAndTolerance_c(const char* name, double value, double tolerance);
 MockExpectedCall_c* withStringParameters_c(const char* name, const char* value);
 MockExpectedCall_c* withPointerParameters_c(const char* name, void* value);
 MockExpectedCall_c* withConstPointerParameters_c(const char* name, const void* value);
@@ -156,12 +160,15 @@ MockExpectedCall_c* withMemoryBufferParameters_c(const char* name, const unsigne
 MockExpectedCall_c* withParameterOfType_c(const char* type, const char* name, const void* value);
 MockExpectedCall_c* withOutputParameterReturning_c(const char* name, const void* value, size_t size);
 MockExpectedCall_c* withOutputParameterOfTypeReturning_c(const char* type, const char* name, const void* value);
+MockExpectedCall_c* withUnmodifiedOutputParameter_c(const char* name);
 MockExpectedCall_c* ignoreOtherParameters_c();
 MockExpectedCall_c* andReturnBoolValue_c(int value);
 MockExpectedCall_c* andReturnIntValue_c(int value);
 MockExpectedCall_c* andReturnUnsignedIntValue_c(unsigned int value);
 MockExpectedCall_c* andReturnLongIntValue_c(long int value);
 MockExpectedCall_c* andReturnUnsignedLongIntValue_c(unsigned long int value);
+MockExpectedCall_c* andReturnLongLongIntValue_c(cpputest_longlong value);
+MockExpectedCall_c* andReturnUnsignedLongLongIntValue_c(cpputest_ulonglong value);
 MockExpectedCall_c* andReturnDoubleValue_c(double value);
 MockExpectedCall_c* andReturnStringValue_c(const char* value);
 MockExpectedCall_c* andReturnPointerValue_c(void* value);
@@ -173,6 +180,8 @@ MockActualCall_c* withActualIntParameters_c(const char* name, int value);
 MockActualCall_c* withActualUnsignedIntParameters_c(const char* name, unsigned int value);
 MockActualCall_c* withActualLongIntParameters_c(const char* name, long int value);
 MockActualCall_c* withActualUnsignedLongIntParameters_c(const char* name, unsigned long int value);
+MockActualCall_c* withActualLongLongIntParameters_c(const char* name, cpputest_longlong value);
+MockActualCall_c* withActualUnsignedLongLongIntParameters_c(const char* name, cpputest_ulonglong value);
 MockActualCall_c* withActualDoubleParameters_c(const char* name, double value);
 MockActualCall_c* withActualStringParameters_c(const char* name, const char* value);
 MockActualCall_c* withActualPointerParameters_c(const char* name, void* value);
@@ -193,6 +202,10 @@ long int longIntReturnValue_c();
 long int returnLongIntValueOrDefault_c(long int defaultValue);
 unsigned long int unsignedLongIntReturnValue_c();
 unsigned long int returnUnsignedLongIntValueOrDefault_c(unsigned long int defaultValue);
+cpputest_longlong longLongIntReturnValue_c();
+cpputest_longlong returnLongLongIntValueOrDefault_c(cpputest_longlong defaultValue);
+cpputest_ulonglong unsignedLongLongIntReturnValue_c();
+cpputest_ulonglong returnUnsignedLongLongIntValueOrDefault_c(cpputest_ulonglong defaultValue);
 const char* stringReturnValue_c();
 const char* returnStringValueOrDefault_c(const char * defaultValue);
 double doubleReturnValue_c();
@@ -237,7 +250,10 @@ static MockExpectedCall_c gExpectedCall = {
         withUnsignedIntParameters_c,
         withLongIntParameters_c,
         withUnsignedLongIntParameters_c,
+        withLongLongIntParameters_c,
+        withUnsignedLongLongIntParameters_c,
         withDoubleParameters_c,
+        withDoubleParametersAndTolerance_c,
         withStringParameters_c,
         withPointerParameters_c,
         withConstPointerParameters_c,
@@ -246,12 +262,15 @@ static MockExpectedCall_c gExpectedCall = {
         withParameterOfType_c,
         withOutputParameterReturning_c,
         withOutputParameterOfTypeReturning_c,
+        withUnmodifiedOutputParameter_c,
         ignoreOtherParameters_c,
         andReturnBoolValue_c,
         andReturnUnsignedIntValue_c,
         andReturnIntValue_c,
         andReturnLongIntValue_c,
         andReturnUnsignedLongIntValue_c,
+        andReturnLongLongIntValue_c,
+        andReturnUnsignedLongLongIntValue_c,
         andReturnDoubleValue_c,
         andReturnStringValue_c,
         andReturnPointerValue_c,
@@ -265,6 +284,8 @@ static MockActualCall_c gActualCall = {
         withActualUnsignedIntParameters_c,
         withActualLongIntParameters_c,
         withActualUnsignedLongIntParameters_c,
+        withActualLongLongIntParameters_c,
+        withActualUnsignedLongLongIntParameters_c,
         withActualDoubleParameters_c,
         withActualStringParameters_c,
         withActualPointerParameters_c,
@@ -286,6 +307,10 @@ static MockActualCall_c gActualCall = {
         returnLongIntValueOrDefault_c,
         unsignedLongIntReturnValue_c,
         returnUnsignedLongIntValueOrDefault_c,
+        longLongIntReturnValue_c,
+        returnLongLongIntValueOrDefault_c,
+        unsignedLongLongIntReturnValue_c,
+        returnUnsignedLongLongIntValueOrDefault_c,
         stringReturnValue_c,
         returnStringValueOrDefault_c,
         doubleReturnValue_c,
@@ -316,6 +341,10 @@ static MockSupport_c gMockSupport = {
         returnLongIntValueOrDefault_c,
         unsignedLongIntReturnValue_c,
         returnUnsignedLongIntValueOrDefault_c,
+        longLongIntReturnValue_c,
+        returnLongLongIntValueOrDefault_c,
+        unsignedLongLongIntReturnValue_c,
+        returnUnsignedLongLongIntValueOrDefault_c,
         stringReturnValue_c,
         returnStringValueOrDefault_c,
         doubleReturnValue_c,
@@ -335,6 +364,7 @@ static MockSupport_c gMockSupport = {
         setConstPointerData_c,
         setFunctionPointerData_c,
         setDataObject_c,
+        setDataConstObject_c,
         getData_c,
         disable_c,
         enable_c,
@@ -378,12 +408,48 @@ MockExpectedCall_c* withUnsignedLongIntParameters_c(const char* name, unsigned l
     return &gExpectedCall;
 }
 
+#ifdef CPPUTEST_USE_LONG_LONG
+
+MockExpectedCall_c* withLongLongIntParameters_c(const char* name, cpputest_longlong value)
+{
+    expectedCall = &expectedCall->withParameter(name, value);
+    return &gExpectedCall;
+}
+
+MockExpectedCall_c* withUnsignedLongLongIntParameters_c(const char* name, cpputest_ulonglong value)
+{
+    expectedCall = &expectedCall->withParameter(name, value);
+    return &gExpectedCall;
+}
+
+#else
+
+MockExpectedCall_c* withLongLongIntParameters_c(const char*, cpputest_longlong)
+{
+    FAIL("Long Long type is not supported");
+    return &gExpectedCall;
+}
+
+MockExpectedCall_c* withUnsignedLongLongIntParameters_c(const char*, cpputest_ulonglong)
+{
+    FAIL("Unsigned Long Long type is not supported");
+    return &gExpectedCall;
+}
+
+#endif
+
 MockExpectedCall_c* withDoubleParameters_c(const char* name, double value)
 {
     expectedCall = &expectedCall->withParameter(name, value);
     return &gExpectedCall;
 }
 
+MockExpectedCall_c* withDoubleParametersAndTolerance_c(const char* name, double value, double tolerance)
+{
+    expectedCall = &expectedCall->withParameter(name, value, tolerance);
+    return &gExpectedCall;
+}
+
 MockExpectedCall_c* withStringParameters_c(const char* name, const char* value)
 {
     expectedCall = &expectedCall->withParameter(name, value);
@@ -432,6 +498,12 @@ MockExpectedCall_c* withOutputParameterOfTypeReturning_c(const char* type, const
     return &gExpectedCall;
 }
 
+MockExpectedCall_c* withUnmodifiedOutputParameter_c(const char* name)
+{
+    expectedCall = &expectedCall->withUnmodifiedOutputParameter(name);
+    return &gExpectedCall;
+}
+
 MockExpectedCall_c* ignoreOtherParameters_c()
 {
     expectedCall = &expectedCall->ignoreOtherParameters();
@@ -468,6 +540,36 @@ MockExpectedCall_c* andReturnUnsignedLongIntValue_c(unsigned long int value)
     return &gExpectedCall;
 }
 
+#ifdef CPPUTEST_USE_LONG_LONG
+
+MockExpectedCall_c* andReturnLongLongIntValue_c(cpputest_longlong value)
+{
+    expectedCall = &expectedCall->andReturnValue(value);
+    return &gExpectedCall;
+}
+
+MockExpectedCall_c* andReturnUnsignedLongLongIntValue_c(cpputest_ulonglong value)
+{
+    expectedCall = &expectedCall->andReturnValue(value);
+    return &gExpectedCall;
+}
+
+#else
+
+MockExpectedCall_c* andReturnLongLongIntValue_c(cpputest_longlong)
+{
+    FAIL("Long Long type is not supported");
+    return &gExpectedCall;
+}
+
+MockExpectedCall_c* andReturnUnsignedLongLongIntValue_c(cpputest_ulonglong)
+{
+    FAIL("Unsigned Long Long type is not supported");
+    return &gExpectedCall;
+}
+
+#endif
+
 MockExpectedCall_c* andReturnDoubleValue_c(double value)
 {
     expectedCall = &expectedCall->andReturnValue(value);
@@ -521,6 +623,16 @@ static MockValue_c getMockValueCFromNamedValue(const MockNamedValue& namedValue)
         returnValue.type = MOCKVALUETYPE_UNSIGNED_LONG_INTEGER;
         returnValue.value.unsignedLongIntValue = namedValue.getUnsignedLongIntValue();
     }
+#ifdef CPPUTEST_USE_LONG_LONG
+    else if (SimpleString::StrCmp(namedValue.getType().asCharString(), "long long int") == 0) {
+        returnValue.type = MOCKVALUETYPE_LONG_LONG_INTEGER;
+        returnValue.value.longLongIntValue = namedValue.getLongLongIntValue();
+    }
+    else if (SimpleString::StrCmp(namedValue.getType().asCharString(), "unsigned long long int") == 0) {
+        returnValue.type = MOCKVALUETYPE_UNSIGNED_LONG_LONG_INTEGER;
+        returnValue.value.unsignedLongLongIntValue = namedValue.getUnsignedLongLongIntValue();
+    }
+#endif
     else if (SimpleString::StrCmp(namedValue.getType().asCharString(), "double") == 0) {
         returnValue.type = MOCKVALUETYPE_DOUBLE;
         returnValue.value.doubleValue = namedValue.getDoubleValue();
@@ -568,7 +680,7 @@ void expectNoCall_c(const char* name)
     currentMockSupport->expectNoCall(name);
 }
 
-MockExpectedCall_c* expectNCalls_c(const int number, const char* name)
+MockExpectedCall_c* expectNCalls_c(const unsigned int number, const char* name)
 {
     expectedCall = &currentMockSupport->expectNCalls(number, name);
     return &gExpectedCall;
@@ -610,6 +722,36 @@ MockActualCall_c* withActualUnsignedLongIntParameters_c(const char* name, unsign
     return &gActualCall;
 }
 
+#ifdef CPPUTEST_USE_LONG_LONG
+
+MockActualCall_c* withActualLongLongIntParameters_c(const char* name, cpputest_longlong value)
+{
+    actualCall = &actualCall->withParameter(name, value);
+    return &gActualCall;
+}
+
+MockActualCall_c* withActualUnsignedLongLongIntParameters_c(const char* name, cpputest_ulonglong value)
+{
+    actualCall = &actualCall->withParameter(name, value);
+    return &gActualCall;
+}
+
+#else
+
+MockActualCall_c* withActualLongLongIntParameters_c(const char*, cpputest_longlong)
+{
+    FAIL("Long Long type is not supported");
+    return &gActualCall;
+}
+
+MockActualCall_c* withActualUnsignedLongLongIntParameters_c(const char*, cpputest_ulonglong)
+{
+    FAIL("Unsigned Long Long type is not supported");
+    return &gActualCall;
+}
+
+#endif
+
 MockActualCall_c* withActualDoubleParameters_c(const char* name, double value)
 {
     actualCall = &actualCall->withParameter(name, value);
@@ -734,6 +876,62 @@ unsigned long int returnUnsignedLongIntValueOrDefault_c(unsigned long int defaul
     return unsignedLongIntReturnValue_c();
 }
 
+#ifdef CPPUTEST_USE_LONG_LONG
+
+cpputest_longlong longLongIntReturnValue_c()
+{
+    return actualCall->returnLongLongIntValue();
+}
+
+cpputest_longlong returnLongLongIntValueOrDefault_c(cpputest_longlong defaultValue)
+{
+    if (!hasReturnValue_c()) {
+        return defaultValue;
+    }
+    return longLongIntReturnValue_c();
+}
+
+cpputest_ulonglong unsignedLongLongIntReturnValue_c()
+{
+    return actualCall->returnUnsignedLongLongIntValue();
+}
+
+cpputest_ulonglong returnUnsignedLongLongIntValueOrDefault_c(cpputest_ulonglong defaultValue)
+{
+    if (!hasReturnValue_c()) {
+        return defaultValue;
+    }
+    return unsignedLongLongIntReturnValue_c();
+}
+
+#else
+
+cpputest_longlong longLongIntReturnValue_c()
+{
+    FAIL("Long Long type is not supported");
+    return cpputest_longlong(0);
+}
+
+cpputest_longlong returnLongLongIntValueOrDefault_c(cpputest_longlong)
+{
+    FAIL("Long Long type is not supported");
+    return cpputest_longlong(0);
+}
+
+cpputest_ulonglong unsignedLongLongIntReturnValue_c()
+{
+    FAIL("Unsigned Long Long type is not supported");
+    return cpputest_ulonglong(0);
+}
+
+cpputest_ulonglong returnUnsignedLongLongIntValueOrDefault_c(cpputest_ulonglong)
+{
+    FAIL("Unsigned Long Long type is not supported");
+    return cpputest_ulonglong(0);
+}
+
+#endif
+
 const char* stringReturnValue_c()
 {
     return actualCall->returnStringValue();
@@ -859,6 +1057,11 @@ void setDataObject_c(const char* name, const char* type, void* value)
     currentMockSupport->setDataObject(name, type, value);
 }
 
+void setDataConstObject_c(const char* name, const char* type, const void* value)
+{
+    currentMockSupport->setDataConstObject(name, type, value);
+}
+
 MockValue_c getData_c(const char* name)
 {
     return getMockValueCFromNamedValue(currentMockSupport->getData(name));
