@@ -138,13 +138,19 @@ void PlatformSpecificFClose(PlatformSpecificFile file) {
 }
 
 extern "C" {
-    
+
 static int IsNanImplementation(double d)
 {
     return isnan(d);
 }
 
+static int IsInfImplementation(double d)
+{
+    return isinf(d);
+}
+
 int (*PlatformSpecificIsNan)(double) = IsNanImplementation;
+int (*PlatformSpecificIsInf)(double) = IsInfImplementation;
 
 }
 
