@@ -39,6 +39,8 @@
 #undef calloc
 #undef realloc
 #undef free
+#undef strdup
+#undef strndup
 
 #define  far  // eliminate "meaningless type qualifier" warning
 
