@@ -73,6 +73,9 @@ static void crashMe ()
 
 TEST(GTestSimpleTest, GTestDeathTest)
 {
+#if defined(GTEST_VERSION_GTEST_1_7)
+    CppuTestGTestIgnoreLeaksInTest();
+#endif
     ASSERT_DEATH(crashMe(), "Crash me!");
 }
 
