@@ -48,6 +48,9 @@ TEST_GROUP(MockCheckedActualCall)
     void teardown()
     {
         CHECK_NO_MOCK_FAILURE();
+
+        MockFailureReporterForTest::clearReporter();
+
         delete emptyList;
         delete list;
     }
@@ -74,12 +77,25 @@ TEST(MockCheckedActualCall, unExpectedCallWithAParameter)
 TEST(MockCheckedActualCall, unExpectedCallWithAnOutputParameter)
 {
     MockCheckedActualCall actualCall(1, reporter, *emptyList);
-    actualCall.withName("unexpected").withOutputParameter("bar", (void*)0);
+    actualCall.withName("unexpected").withOutputParameter("bar", NULLPTR);
 
     MockUnexpectedCallHappenedFailure expectedFailure(mockFailureTest(), "unexpected", *list);
     CHECK_EXPECTED_MOCK_FAILURE(expectedFailure);
 }
 
+TEST(MockCheckedActualCall, unExpectedCallOnObject)
+{
+    int object;
+
+    MockCheckedActualCall actualCall(1, reporter, *emptyList);
+    actualCall.withName("unexpected").onObject(&object);
+
+    MockUnexpectedCallHappenedFailure expectedFailure(mockFailureTest(), "unexpected", *list);
+    CHECK_EXPECTED_MOCK_FAILURE(expectedFailure);
+
+    CHECK(actualCall.hasFailed()); // Checks that onObject() doesn't "reset" call state
+}
+
 TEST(MockCheckedActualCall, actualCallWithNoReturnValueAndMeaninglessCallOrderForCoverage)
 {
     MockCheckedActualCall actualCall(1, reporter, *emptyList);
@@ -138,7 +154,8 @@ TEST(MockCheckedActualCall, MockIgnoredActualCallWorksAsItShould)
     actual.withCallOrder(1);
 
     CHECK(false == actual.returnBoolValue());
-    CHECK(false == actual.returnBoolValueOrDefault(true));
+    CHECK(true == actual.returnBoolValueOrDefault(true));
+    CHECK(false == actual.returnBoolValueOrDefault(false));
     CHECK(0 == actual.returnUnsignedLongIntValue());
     CHECK(0 == actual.returnIntValue());
     CHECK(1ul == actual.returnUnsignedLongIntValueOrDefault(1ul));
@@ -147,15 +164,21 @@ TEST(MockCheckedActualCall, MockIgnoredActualCallWorksAsItShould)
     CHECK(1l == actual.returnLongIntValueOrDefault(1l));
     CHECK(0 == actual.returnUnsignedIntValue());
     CHECK(1u == actual.returnUnsignedIntValueOrDefault(1u));
+#ifdef CPPUTEST_USE_LONG_LONG
+    CHECK(0 == actual.returnLongLongIntValue());
+    CHECK(1ll == actual.returnLongLongIntValueOrDefault(1ll));
+    CHECK(0 == actual.returnUnsignedLongLongIntValue());
+    CHECK(1ull == actual.returnUnsignedLongLongIntValueOrDefault(1ull));
+#endif
     DOUBLES_EQUAL(0.0, actual.returnDoubleValue(), 0.0);
     DOUBLES_EQUAL(1.5, actual.returnDoubleValueOrDefault(1.5), 0.0);
     STRCMP_EQUAL("bla", actual.returnStringValueOrDefault("bla"));
     STRCMP_EQUAL("", actual.returnStringValue());
-    CHECK(0 == actual.returnPointerValue());
+    CHECK(NULLPTR == actual.returnPointerValue());
     CHECK((void*) 0x2 == actual.returnPointerValueOrDefault((void*) 0x2));
-    CHECK(0 == actual.returnConstPointerValue());
+    CHECK(NULLPTR == actual.returnConstPointerValue());
     CHECK((const void*) 0x2 == actual.returnConstPointerValueOrDefault((const void*) 0x2));
-    CHECK(0 == actual.returnFunctionPointerValue());
+    CHECK(NULLPTR == actual.returnFunctionPointerValue());
     CHECK((void(*)()) 1 == actual.returnFunctionPointerValueOrDefault((void(*)()) 0x1));
     CHECK_FALSE(actual.hasReturnValue());
     CHECK(actual.returnValue().equals(MockNamedValue("")));
@@ -176,6 +199,10 @@ TEST(MockCheckedActualCall, remainderOfMockActualCallTraceWorksAsItShould)
     actual.withUnsignedIntParameter("unsigned_int", (unsigned int) 1);
     actual.withUnsignedLongIntParameter("unsigned_long", (unsigned long)1);
     actual.withLongIntParameter("long_int", (long int) 1);
+#ifdef CPPUTEST_USE_LONG_LONG
+    actual.withLongLongIntParameter("long_long_int", (long long int) 1);
+    actual.withUnsignedLongLongIntParameter("unsigned_long_long_int", (unsigned long long int) 1);
+#endif
     actual.withPointerParameter("pointer", &value);
     actual.withConstPointerParameter("const_pointer", &const_value);
     actual.withFunctionPointerParameter("function_pointer", function_value);
@@ -187,9 +214,13 @@ TEST(MockCheckedActualCall, remainderOfMockActualCallTraceWorksAsItShould)
     expectedString += " onObject:0x";
     expectedString += HexStringFrom(&value);
     expectedString += " bool:true";
-    expectedString += " unsigned_int:         1 (0x00000001)";
+    expectedString += " unsigned_int:1 (0x1)";
     expectedString += " unsigned_long:1 (0x1)";
-    expectedString += " long_int:1";
+    expectedString += " long_int:1 (0x1)";
+#ifdef CPPUTEST_USE_LONG_LONG
+    expectedString += " long_long_int:1 (0x1)";
+    expectedString += " unsigned_long_long_int:1 (0x1)";
+#endif
     expectedString += " pointer:0x";
     expectedString += HexStringFrom(&value);
     expectedString += " const_pointer:0x";
@@ -212,17 +243,30 @@ TEST(MockCheckedActualCall, remainderOfMockActualCallTraceWorksAsItShould)
     CHECK(0 == actual.returnIntValueOrDefault(1));
     CHECK(0 == actual.returnLongIntValue());
     CHECK(0 == actual.returnLongIntValueOrDefault(1l));
+#ifdef CPPUTEST_USE_LONG_LONG
+    CHECK(0 == actual.returnLongLongIntValue());
+    CHECK(0 == actual.returnLongLongIntValueOrDefault(1ll));
+    CHECK(0 == actual.returnUnsignedLongLongIntValue());
+    CHECK(0 == actual.returnUnsignedLongLongIntValueOrDefault(1ull));
+#endif
     CHECK(0 == actual.returnUnsignedIntValue());
     CHECK(0 == actual.returnUnsignedIntValueOrDefault(1u));
     DOUBLES_EQUAL(0.0, actual.returnDoubleValue(), 0.0);
     DOUBLES_EQUAL(0.0, actual.returnDoubleValueOrDefault(1.0), 0.0);
     STRCMP_EQUAL("", actual.returnStringValueOrDefault("bla"));
     STRCMP_EQUAL("", actual.returnStringValue());
-    CHECK(0 == actual.returnPointerValue());
-    CHECK(0 == actual.returnPointerValueOrDefault((void*) 0x0));
-    CHECK(0 == actual.returnConstPointerValue());
-    CHECK(0 == actual.returnConstPointerValueOrDefault((const void*) 0x0));
-    CHECK(0 == actual.returnFunctionPointerValue());
-    CHECK(0 == actual.returnFunctionPointerValueOrDefault((void (*)()) 0x0));
+    CHECK(NULLPTR == actual.returnPointerValue());
+    CHECK(NULLPTR == actual.returnPointerValueOrDefault((void*) NULLPTR));
+    CHECK(NULLPTR == actual.returnConstPointerValue());
+    CHECK(NULLPTR == actual.returnConstPointerValueOrDefault((const void*) NULLPTR));
+    CHECK(NULLPTR == actual.returnFunctionPointerValue());
+    CHECK(NULLPTR == actual.returnFunctionPointerValueOrDefault((void (*)()) NULLPTR));
 }
 
+TEST(MockCheckedActualCall, MockActualCallTraceClear)
+{
+    MockActualCallTrace actual;
+    actual.withName("func");
+    actual.clear();
+    STRCMP_EQUAL("", actual.getTraceOutput());
+}
