@@ -54,7 +54,9 @@ TEST_GROUP(MockFailureTest)
         delete call2;
         delete call3;
         CHECK_NO_MOCK_FAILURE();
+        MockFailureReporterForTest::clearReporter();
     }
+
     void addAllToList()
     {
         list->addExpectedCall(call1);
@@ -65,12 +67,22 @@ TEST_GROUP(MockFailureTest)
     void checkUnexpectedNthCallMessage(unsigned int count, const char* expectedOrdinal)
     {
         MockExpectedCallsList callList;
-        MockCheckedExpectedCall expCall;
+        MockCheckedExpectedCall expectedCallSingle(1);
+        MockCheckedExpectedCall expectedCallMulti(count-1);
 
-        expCall.withName("bar");
-        for (unsigned int i = 0; i < (count - 1); i++) {
-            expCall.callWasMade(1);
-            callList.addExpectedCall(&expCall);
+        expectedCallSingle.withName("bar");
+        expectedCallMulti.withName("bar");
+
+        if (count > 1) {
+            callList.addExpectedCall(&expectedCallSingle);
+            expectedCallSingle.callWasMade(1);
+        }
+
+        if (count > 2) {
+            callList.addExpectedCall(&expectedCallMulti);
+            for (unsigned int i = 1; i < (count - 1); i++) {
+                expectedCallMulti.callWasMade(i+1);
+            }
         }
 
         MockUnexpectedCallHappenedFailure failure(UtestShell::getCurrent(), "bar", callList);
@@ -90,9 +102,9 @@ TEST(MockFailureTest, unexpectedCallHappened)
 {
     MockUnexpectedCallHappenedFailure failure(UtestShell::getCurrent(), "foobar", *list);
     STRCMP_EQUAL("Mock Failure: Unexpected call to function: foobar\n"
-                 "\tEXPECTED calls that did NOT happen:\n"
+                 "\tEXPECTED calls that WERE NOT fulfilled:\n"
                  "\t\t<none>\n"
-                 "\tACTUAL calls that did happen (in call order):\n"
+                 "\tEXPECTED calls that WERE fulfilled:\n"
                  "\t\t<none>", failure.getMessage().asCharString());
 }
 
@@ -105,12 +117,12 @@ TEST(MockFailureTest, expectedCallDidNotHappen)
     addAllToList();
 
     MockExpectedCallsDidntHappenFailure failure(UtestShell::getCurrent(), *list);
-    STRCMP_EQUAL("Mock Failure: Expected call did not happen.\n"
-                 "\tEXPECTED calls that did NOT happen:\n"
-                 "\t\tfoobar -> no parameters\n"
-                 "\t\tworld -> int boo: <2>, const char* hello: <world>\n"
-                 "\tACTUAL calls that did happen (in call order):\n"
-                 "\t\thaphaphap -> no parameters", failure.getMessage().asCharString());
+    STRCMP_EQUAL("Mock Failure: Expected call WAS NOT fulfilled.\n"
+                 "\tEXPECTED calls that WERE NOT fulfilled:\n"
+                 "\t\tfoobar -> no parameters (expected 1 call, called 0 times)\n"
+                 "\t\tworld -> int boo: <2 (0x2)>, const char* hello: <world> (expected 1 call, called 0 times)\n"
+                 "\tEXPECTED calls that WERE fulfilled:\n"
+                 "\t\thaphaphap -> no parameters (expected 1 call, called 1 time)", failure.getMessage().asCharString());
 }
 
 TEST(MockFailureTest, MockUnexpectedNthAdditionalCallFailure)
@@ -139,13 +151,13 @@ TEST(MockFailureTest, MockUnexpectedInputParameterFailure)
 
     MockUnexpectedInputParameterFailure failure(UtestShell::getCurrent(), "foo", actualParameter, *list);
     STRCMP_EQUAL("Mock Failure: Unexpected parameter name to function \"foo\": bar\n"
-                 "\tEXPECTED calls that DID NOT happen related to function: foo\n"
-                 "\t\tfoo -> int boo: <2>\n"
-                 "\t\tfoo -> double boo: <3.3>\n"
-                 "\tACTUAL calls that DID happen related to function: foo\n"
+                 "\tEXPECTED calls that WERE NOT fulfilled related to function: foo\n"
+                 "\t\tfoo -> int boo: <2 (0x2)> (expected 1 call, called 0 times)\n"
+                 "\t\tfoo -> double boo: <3.3> (expected 1 call, called 0 times)\n"
+                 "\tEXPECTED calls that WERE fulfilled related to function: foo\n"
                  "\t\t<none>\n"
                  "\tACTUAL unexpected parameter passed to function: foo\n"
-                 "\t\tint bar: <2>", failure.getMessage().asCharString());
+                 "\t\tint bar: <2 (0x2)>", failure.getMessage().asCharString());
 }
 
 TEST(MockFailureTest, MockUnexpectedOutputParameterFailure)
@@ -162,10 +174,32 @@ TEST(MockFailureTest, MockUnexpectedOutputParameterFailure)
 
     MockUnexpectedOutputParameterFailure failure(UtestShell::getCurrent(), "foo", actualParameter, *list);
     STRCMP_EQUAL("Mock Failure: Unexpected output parameter name to function \"foo\": bar\n"
-                 "\tEXPECTED calls that DID NOT happen related to function: foo\n"
-                 "\t\tfoo -> const void* boo: <output>\n"
-                 "\t\tfoo -> const void* boo: <output>\n"
-                 "\tACTUAL calls that DID happen related to function: foo\n"
+                 "\tEXPECTED calls that WERE NOT fulfilled related to function: foo\n"
+                 "\t\tfoo -> const void* boo: <output> (expected 1 call, called 0 times)\n"
+                 "\t\tfoo -> const void* boo: <output> (expected 1 call, called 0 times)\n"
+                 "\tEXPECTED calls that WERE fulfilled related to function: foo\n"
+                 "\t\t<none>\n"
+                 "\tACTUAL unexpected output parameter passed to function: foo\n"
+                 "\t\tvoid* bar", failure.getMessage().asCharString());
+}
+
+TEST(MockFailureTest, MockUnexpectedUnmodifiedOutputParameterFailure)
+{
+    int out1;
+    call1->withName("foo").withOutputParameterReturning("boo", &out1, sizeof(out1));
+    call2->withName("foo").withUnmodifiedOutputParameter("boo");
+    call3->withName("unrelated");
+    addAllToList();
+
+    MockNamedValue actualParameter("bar");
+    actualParameter.setValue((void *)0x123);
+
+    MockUnexpectedOutputParameterFailure failure(UtestShell::getCurrent(), "foo", actualParameter, *list);
+    STRCMP_EQUAL("Mock Failure: Unexpected output parameter name to function \"foo\": bar\n"
+                 "\tEXPECTED calls that WERE NOT fulfilled related to function: foo\n"
+                 "\t\tfoo -> const void* boo: <output> (expected 1 call, called 0 times)\n"
+                 "\t\tfoo -> const void* boo: <output> (expected 1 call, called 0 times)\n"
+                 "\tEXPECTED calls that WERE fulfilled related to function: foo\n"
                  "\t\t<none>\n"
                  "\tACTUAL unexpected output parameter passed to function: foo\n"
                  "\t\tvoid* bar", failure.getMessage().asCharString());
@@ -182,14 +216,14 @@ TEST(MockFailureTest, MockUnexpectedParameterValueFailure)
     actualParameter.setValue(20);
 
     MockUnexpectedInputParameterFailure failure(UtestShell::getCurrent(), "foo", actualParameter, *list);
-    STRCMP_EQUAL("Mock Failure: Unexpected parameter value to parameter \"boo\" to function \"foo\": <20>\n"
-                 "\tEXPECTED calls that DID NOT happen related to function: foo\n"
-                 "\t\tfoo -> int boo: <2>\n"
-                 "\t\tfoo -> int boo: <10>\n"
-                 "\tACTUAL calls that DID happen related to function: foo\n"
+    STRCMP_EQUAL("Mock Failure: Unexpected parameter value to parameter \"boo\" to function \"foo\": <20 (0x14)>\n"
+                 "\tEXPECTED calls that WERE NOT fulfilled related to function: foo\n"
+                 "\t\tfoo -> int boo: <2 (0x2)> (expected 1 call, called 0 times)\n"
+                 "\t\tfoo -> int boo: <10 (0xa)> (expected 1 call, called 0 times)\n"
+                 "\tEXPECTED calls that WERE fulfilled related to function: foo\n"
                  "\t\t<none>\n"
                  "\tACTUAL unexpected parameter passed to function: foo\n"
-                 "\t\tint boo: <20>", failure.getMessage().asCharString());
+                 "\t\tint boo: <20 (0x14)>", failure.getMessage().asCharString());
 }
 
 TEST(MockFailureTest, MockExpectedParameterDidntHappenFailure)
@@ -204,10 +238,10 @@ TEST(MockFailureTest, MockExpectedParameterDidntHappenFailure)
 
     MockExpectedParameterDidntHappenFailure failure(UtestShell::getCurrent(), "foo", *list);
     STRCMP_EQUAL("Mock Failure: Expected parameter for function \"foo\" did not happen.\n"
-                 "\tEXPECTED calls that DID NOT happen related to function: foo\n"
-                 "\t\tfoo -> int bar: <2>, const char* boo: <str>\n"
-                 "\tACTUAL calls that DID happen related to function: foo\n"
-                 "\t\tfoo -> int bar: <10>, const char* boo: <bleh>\n"
+                 "\tEXPECTED calls that WERE NOT fulfilled related to function: foo\n"
+                 "\t\tfoo -> int bar: <2 (0x2)>, const char* boo: <str> (expected 1 call, called 0 times)\n"
+                 "\tEXPECTED calls that WERE fulfilled related to function: foo\n"
+                 "\t\tfoo -> int bar: <10 (0xa)>, const char* boo: <bleh> (expected 1 call, called 1 time)\n"
                  "\tMISSING parameters that didn't happen:\n"
                  "\t\tint bar, const char* boo", failure.getMessage().asCharString());
 }
@@ -231,10 +265,10 @@ TEST(MockFailureTest, MockUnexpectedObjectFailure)
     STRCMP_EQUAL(StringFromFormat (
                  "MockFailure: Function called on an unexpected object: foo\n"
                  "\tActual object for call has address: <%p>\n"
-                 "\tEXPECTED calls that DID NOT happen related to function: foo\n"
-                 "\t\t(object address: %p)::foo -> no parameters\n"
-                 "\tACTUAL calls that DID happen related to function: foo\n"
-                 "\t\t(object address: %p)::foo -> no parameters",
+                 "\tEXPECTED calls that WERE NOT fulfilled related to function: foo\n"
+                 "\t\t(object address: %p)::foo -> no parameters (expected 1 call, called 0 times)\n"
+                 "\tEXPECTED calls that WERE fulfilled related to function: foo\n"
+                 "\t\t(object address: %p)::foo -> no parameters (expected 1 call, called 1 time)",
                  (void*) 0x01, (void*) 0x02, (void*) 0x03).asCharString(), failure.getMessage().asCharString());
 }
 
@@ -250,9 +284,9 @@ TEST(MockFailureTest, MockExpectedObjectDidntHappenFailure)
     MockExpectedObjectDidntHappenFailure failure(UtestShell::getCurrent(), "foo", *list);
     STRCMP_EQUAL(StringFromFormat(
                  "Mock Failure: Expected call on object for function \"foo\" but it did not happen.\n"
-                 "\tEXPECTED calls that DID NOT happen related to function: foo\n"
-                 "\t\t(object address: %p)::foo -> no parameters\n"
-                 "\tACTUAL calls that DID happen related to function: foo\n"
-                 "\t\t(object address: %p)::foo -> no parameters",
+                 "\tEXPECTED calls that WERE NOT fulfilled related to function: foo\n"
+                 "\t\t(object address: %p)::foo -> no parameters (expected 1 call, called 0 times)\n"
+                 "\tEXPECTED calls that WERE fulfilled related to function: foo\n"
+                 "\t\t(object address: %p)::foo -> no parameters (expected 1 call, called 1 time)",
                  (void*) 0x2, (void*) 0x3).asCharString(), failure.getMessage().asCharString());
 }
