@@ -53,7 +53,13 @@ void all_mock_support_c_calls(void)
             withUnsignedIntParameters("unsigned", 1)->
             withLongIntParameters("long int", (long int) -1)->
             withUnsignedLongIntParameters("unsigned long int", (unsigned long int) 1)->
+#ifdef CPPUTEST_USE_LONG_LONG
+            withLongLongIntParameters("long long int", (long long int) -1)->
+            withUnsignedLongLongIntParameters("unsigned long long int", (unsigned long long int) 1)->
+#endif
+
             withDoubleParameters("double", 1.0)->
+            withDoubleParametersAndTolerance("doubleWithTolerance", 1.0, 1.0)->
             withStringParameters("string", "string")->
             withPointerParameters("pointer", (void*) 1)->
             withConstPointerParameters("constpointer", (const void*) 1)->
@@ -66,7 +72,12 @@ void all_mock_support_c_calls(void)
             withUnsignedIntParameters("unsigned", 1)->
             withLongIntParameters("long int", (long int) -1)->
             withUnsignedLongIntParameters("unsigned long int", (unsigned long int) 1)->
+#ifdef CPPUTEST_USE_LONG_LONG
+            withLongLongIntParameters("long long int", (long long int) -1)->
+            withUnsignedLongLongIntParameters("unsigned long long int", (unsigned long long int) 1)->
+#endif
             withDoubleParameters("double", 1.0)->
+            withDoubleParameters("doubleWithTolerance", 0.0 )->
             withStringParameters("string", "string")->
             withPointerParameters("pointer", (void*) 1)->
             withConstPointerParameters("constpointer", (const void*) 1)->
@@ -100,18 +111,28 @@ void all_mock_support_c_calls(void)
     mock_c()->intReturnValue();
     mock_c()->returnValue();
 
-    mock_c()->expectOneCall("boo2")->andReturnUnsignedIntValue(1.0);
+    mock_c()->expectOneCall("boo2")->andReturnUnsignedIntValue(1);
     mock_c()->actualCall("boo2")->unsignedIntReturnValue();
     mock_c()->unsignedIntReturnValue();
 
-    mock_c()->expectOneCall("boo3")->andReturnLongIntValue(1.0);
+    mock_c()->expectOneCall("boo3")->andReturnLongIntValue(1);
     mock_c()->actualCall("boo3")->longIntReturnValue();
     mock_c()->longIntReturnValue();
 
-    mock_c()->expectOneCall("boo3")->andReturnUnsignedLongIntValue(1.0);
+    mock_c()->expectOneCall("boo3")->andReturnUnsignedLongIntValue(1);
     mock_c()->actualCall("boo3")->unsignedLongIntReturnValue();
     mock_c()->unsignedLongIntReturnValue();
 
+#ifdef CPPUTEST_USE_LONG_LONG
+    mock_c()->expectOneCall("mgrgrgr1")->andReturnLongLongIntValue(1);
+    mock_c()->actualCall("mgrgrgr1")->longLongIntReturnValue();
+    mock_c()->longLongIntReturnValue();
+
+    mock_c()->expectOneCall("mgrgrgr2")->andReturnUnsignedLongLongIntValue(1);
+    mock_c()->actualCall("mgrgrgr2")->unsignedLongLongIntReturnValue();
+    mock_c()->unsignedLongLongIntReturnValue();
+#endif
+
     mock_c()->expectOneCall("boo4")->andReturnDoubleValue(1.0);
     mock_c()->actualCall("boo4")->doubleReturnValue();
     mock_c()->doubleReturnValue();
@@ -168,6 +189,10 @@ void all_mock_support_c_calls(void)
     mock_c()->returnUnsignedIntValueOrDefault(1);
     mock_c()->returnLongIntValueOrDefault(-1L);
     mock_c()->returnUnsignedLongIntValueOrDefault(1L);
+#ifdef CPPUTEST_USE_LONG_LONG
+    mock_c()->returnLongLongIntValueOrDefault(-1LL);
+    mock_c()->returnUnsignedLongLongIntValueOrDefault(1ULL);
+#endif
     mock_c()->returnStringValueOrDefault("");
     mock_c()->returnDoubleValueOrDefault(0.01);
     mock_c()->returnPointerValueOrDefault(0);
